@@ -512,7 +512,7 @@ ghex_window_new(void)
     win->uic = uic;
 	bonobo_ui_component_set_container(uic, BONOBO_OBJREF(ui_container),
                                       NULL);
-	bonobo_ui_util_set_ui(uic, NULL, "ghex-ui.xml", "GHex", NULL);
+	bonobo_ui_util_set_ui(uic, DATADIR, "ghex-ui.xml", "GHex", NULL);
 	bonobo_ui_component_add_verb_list_with_data(uic, ghex_verbs, win);
     
 	bonobo_ui_component_add_listener (uic, "Bytes",
@@ -651,7 +651,9 @@ ghex_window_update_status_message(GHexWindow *win)
                           offset_fmt, offset_fmt, offset_fmt) < FMT_LEN) {
                 len = strlen(status);
                 if(len < STATUS_LEN) {
-                    g_snprintf(status + len, STATUS_LEN - len, fmt, se - ss, ss, se);
+                    // Variables 'ss' and 'se' denotes the offsets of the first and
+                    // the last bytes that are part of the selection.
+                    g_snprintf(status + len, STATUS_LEN - len, fmt, se - ss + 1, ss, se);
                 }
             }
         }
@@ -664,7 +666,6 @@ ghex_window_update_status_message(GHexWindow *win)
 static void
 cursor_moved_cb(GtkHex *gtkhex, gpointer user_data)
 {
-	static gchar *cursor_pos, *format;
     int i;
     int current_pos;
     HexDialogVal64 val;
