@@ -673,7 +673,7 @@ static void render_offsets(GtkHex *gh, gint imin, gint imax) {
 	gdk_gc_set_foreground(gh->offsets_gc, &GTK_WIDGET(gh)->style->text[GTK_STATE_NORMAL]);
 	
 	for(i = imin; i <= imax; i++) {
-		sprintf(offstr, "%08X", (gh->top_line + i)*gh->cpl);
+		sprintf(offstr, "%08X", (gh->top_line + i)*gh->cpl + gh->starting_offset);
 		/* Changes for Gnome 2.0 */
 		pango_layout_set_text (gh->olayout, offstr, 8);
 		gdk_draw_layout (w->window, gh->offsets_gc, 0, i*gh->char_height, gh->olayout);
@@ -730,9 +730,6 @@ static void draw_shadow(GtkWidget *widget, GdkRectangle *area) {
 	GtkHex *gh = GTK_HEX(widget);
 	gint border = GTK_CONTAINER(widget)->border_width;
 	gint x;
-	
-	gdk_window_clear_area (widget->window, area->x, area->y,
-						   area->width, area->height);
 
 	x = border;
 	if(gh->show_offsets) {
@@ -1208,6 +1205,10 @@ static void gtk_hex_real_data_changed(GtkHex *gh, gpointer data) {
 
 	render_hex_lines(gh, start_line, end_line);
 	render_ascii_lines(gh, start_line, end_line);
+    if (gh->show_offsets)
+    {
+        render_offsets (gh, start_line, end_line);
+    }
 }
 
 static void bytes_changed(GtkHex *gh, gint start, gint end)
@@ -1223,6 +1224,10 @@ static void bytes_changed(GtkHex *gh, gint start, gint end)
 
 	render_hex_lines(gh, start_line, end_line);
 	render_ascii_lines(gh, start_line, end_line);
+    if (gh->show_offsets)
+    {
+        render_offsets (gh, start_line, end_line);
+    }
 }
 
 static void primary_get_cb(GtkClipboard *clipboard,
@@ -1602,7 +1607,6 @@ static gboolean gtk_hex_key_press(GtkWidget *w, GdkEventKey *event) {
 	GtkHex *gh = GTK_HEX(w);
 	guint old_cp = gh->cursor_pos;
 	gint ret = TRUE;
-	gint cx, cy;
 
 	hide_cursor(gh);
 
@@ -1847,6 +1851,13 @@ static gint gtk_hex_expose(GtkWidget *w, GdkEventExpose *event) {
 	return TRUE;
 }
 
+static void gtk_hex_document_changed(HexDocument* doc, gpointer change_data,
+        gboolean push_undo, gpointer data)
+{
+    gtk_hex_real_data_changed (GTK_HEX(data), change_data);
+}
+
+
 static void gtk_hex_size_request(GtkWidget *w, GtkRequisition *req) {
 	GtkHex *gh = GTK_HEX(w);
 	GtkRequisition sb_req;
@@ -1936,6 +1947,7 @@ static void gtk_hex_init(GtkHex *gh, gpointer klass) {
 
 	gh->disp_buffer = NULL;
 	gh->document = NULL;
+	gh->starting_offset = 0;
 
 	gh->xdisp_width = gh->adisp_width = 200;
 	gh->adisp_gc = gh->xdisp_gc = NULL;
@@ -2028,8 +2040,8 @@ static void gtk_hex_init(GtkHex *gh, gpointer klass) {
 	gtk_widget_show(gh->scrollbar);
 }
 
-guint gtk_hex_get_type() {
-	static guint gh_type = 0;
+GType gtk_hex_get_type() {
+	static GType gh_type = 0;
 	
 	if(!gh_type) {
 		GTypeInfo gh_info = {
@@ -2060,6 +2072,8 @@ GtkWidget *gtk_hex_new(HexDocument *owner) {
 	g_return_val_if_fail (gh != NULL, NULL);
 
 	gh->document = owner;
+    g_signal_connect (G_OBJECT (gh->document), "document_changed",
+            G_CALLBACK (gtk_hex_document_changed), gh);
 	
 	return GTK_WIDGET(gh);
 }
@@ -2130,18 +2144,19 @@ void gtk_hex_set_cursor(GtkHex *gh, gint index) {
 		if(index == gh->document->file_size)
 			gh->lower_nibble = FALSE;
 		
-		g_signal_emit_by_name(G_OBJECT(gh), "cursor_moved");
-
 		if(gh->selecting) {
 			gtk_hex_set_selection(gh, gh->selection.start, gh->cursor_pos);
 			bytes_changed(gh, MIN(gh->cursor_pos, old_pos), MAX(gh->cursor_pos, old_pos));
 		}
-		else if(gh->selection.end != gh->selection.start) {
+		else {
 			guint start = MIN(gh->selection.start, gh->selection.end);
 			guint end = MAX(gh->selection.start, gh->selection.end);
-			gh->selection.end = gh->selection.start = 0;
 			bytes_changed(gh, start, end);
+			gh->selection.end = gh->selection.start = gh->cursor_pos;
 		}
+
+		g_signal_emit_by_name(G_OBJECT(gh), "cursor_moved");
+
 		bytes_changed(gh, old_pos, old_pos);
 		show_cursor(gh);
 	}
@@ -2237,7 +2252,7 @@ void gtk_hex_set_group_type(GtkHex *gh, guint gt) {
 /*
  * sets font for displaying data
  */
-void gtk_hex_set_font(GtkHex *gh, PangoFontMetrics *font_metrics, PangoFontDescription *font_desc) {
+void gtk_hex_set_font(GtkHex *gh, const PangoFontMetrics *font_metrics, const PangoFontDescription *font_desc) {
 	g_return_if_fail(gh != NULL);
 	g_return_if_fail(GTK_IS_HEX(gh));
 
@@ -2285,6 +2300,13 @@ void gtk_hex_show_offsets(GtkHex *gh, gboolean show)
 		hide_offsets_widget(gh);
 }
 
+void gtk_hex_set_starting_offset(GtkHex *gh, gint starting_offset)
+{
+	g_return_if_fail (gh != NULL);
+	g_return_if_fail(GTK_IS_HEX(gh));
+	gh->starting_offset = starting_offset;
+}
+
 void gtk_hex_set_insert_mode(GtkHex *gh, gboolean insert)
 {
 	g_return_if_fail(gh != NULL);
